/*
Copyright (C) 2017 Martijn Braam & Clayton Craft <clayton@craftyguy.net>

This file is part of osk-sdl.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UTIL_H
#define UTIL_H
#include "config.h"
#include "keyboard.h"
#include "luksdevice.h"
#include <SDL2/SDL.h>
#include <iostream>
#include <list>
#include <math.h>
#include <string>
#include <unistd.h>

const std::string DEFAULT_LUKSDEVPATH = "/home/user/disk";
const std::string DEFAULT_LUKSDEVNAME = "root";
const std::string DEFAULT_CONFPATH = "/etc/osk.conf";

struct Opts {
	std::string luksDevPath;
	std::string luksDevName;
	std::string confPath;
	bool testMode;
	bool verbose;
};

/**
  Fetch command line options
  @param argc Number of arguments passed on cmdline
  @param args Arguments from cmdline
  @param opts Structure for storing options from cmdline
  @return 0 on success, non-zero on failure
 */
int fetchOpts(int argc, char **args, Opts *opts);

/**
  Convert list of strings into a single string
  @param strList List of strings
  @return String with all elements of strList concatenated together
 */
std::string strList2str(const std::list<std::string> *strList);

/**
  Create wallpaper
  @param renderer Initialized SDL_Renderer
  @param config Config paramters
  @param width Width of wallpaper to generate
  @param height Height of wallpaper to generate
  @return Initialized SDL_Surface, else NULL on failure
 */
SDL_Surface *make_wallpaper(SDL_Renderer *renderer, Config *config,
	int width, int height);

/**
  Draw a circle
  @param renderer Initialized SDL_Renderer object
  @param center Center position of circle
  @param radius Radius of circle
 */
void draw_circle(SDL_Renderer *renderer, SDL_Point center, int radius);

/**
  Handle keypresses for virtual keyboard
  @param tapped Character tapped on keyboard
  @param kbd Initialized Keyboard obj
  @param lkd Initialized LuksDevice obj
  @param lkd passphrase Passphrase to modify
 */
void handleVirtualKeyPress(std::string tapped, Keyboard *kbd, LuksDevice *lkd, std::list<std::string> *passphrase);

/**
  Draw the dots to represent hidden characters
  @param renderer Initialized SDL_Renderer object
  @param inputHeight Height of input box
  @param screenWidth Width of overall screen
  @param numDots Number of password 'dots' to draw
  @param y Vertical position of the input box
  @param busy if true the dots will play a loading animation
 */
void draw_password_box_dots(SDL_Renderer *renderer, Config *config, int inputHeight,
	int screenWidth, int numDots, int y, bool busy);
#endif
